/*
 *	Copyright 2018 Scott Cooper
 *
 *	This program is free software: you can redistribute it and/or modify it under the terms
 *	of the GNU General Public License as published by the Free Software Foundation,
 *	either version 2 of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *	See the GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along with this program.
 *	If not, see <http://www.gnu.org/licenses/>.
 */

/*
 *	This is a cremation function for Glen the Undertaker in U6.
 *
 *	2018-12-23 Knight Captain
 */

void greenCremation object#(0x9D6) ()
{
	var body_400_frames;
	var body_402_frames;
	var body_414_frames;
	var body_507_frames;
	var body_762_frames;
	var body_892_frames;
	var total_count;
	var body_count;
	var each;
	var body_400_shape;
	var body_402_shape;
	var body_414_shape;
	var body_507_shape;
	var body_762_shape;
	var body_892_shape;
	var bodies;
	var index;
	var max;
	var frame;
	var frame_index;
	var frame_max;
	var valid_bodies;
	
	const int SHAPE_BODY1 = 400; 
	const int SHAPE_BODY2 = 414;
	const int SHAPE_BODY3 = 762;
	const int SHAPE_BODY4 = 892; //need to limit by frames
	var body_frames4 = [1, 2, 3, 4, 5, 6, 7, 10, 11, 29];
	const int SHAPE_BODY5 = 1140;


	UI_error_message(""); // Blank line.
	UI_error_message("greenCremation starting.");

	///////////////////////////////////////////////////////////////////////////
	// Since some frames of each body shape either cannot or should not be cremated,
	// we have to list individual frames. These are SI-specific.

	// Excludes Petra and 29 which is blank.
	body_400_frames = ([1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
						11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 
						21, 22, 23, 24, 25, 26, 27, 28]); 
	UI_error_message("greenCremation body_400_frames are ", body_400_frames, ".");

	// Excludes monsters, undead, and brass automaton, as well as 24 which is blank.
	body_402_frames = ([1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
						11, 12, 14, 15, 20,
						22, 23, 26, 27, 28, 30]);
	UI_error_message("greenCremation body_402_frames are ", body_402_frames, ".");

	// Excludes party members, Cantra, and 26 which is blank.
	body_414_frames = ([1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
						11, 12, 13, 14, 15, 16, 17, 
						22, 23, 24, 25, 27, 28, 29, 30, 31]); 
	UI_error_message("greenCremation body_414_frames are ", body_414_frames, ".");

	// Torsos, Gwani body frames, the rest are skeletons.
	body_507_frames = ([11, 12, 14, 15, 16]);
	UI_error_message("greenCremation body_507_frames are ", body_507_frames, ".");

	// Most frames are monsters or dead Banes.
	body_762_frames = ([5, 6, 8, 9, 11, 22, 23, 24]);
	UI_error_message("greenCremation body_762_frames are ", body_762_frames, ".");

	// Most frames are monsters or dead Banes.
	body_892_frames = ([23, 29, 30]);
	UI_error_message("greenCremation body_892_frames are ", body_892_frames, ".");

	///////////////////////////////////////////////////////////////////////////
	// Let's check if there are any bodies at all.
	total_count = (UI_count_objects(PARTY, SHAPE_BODY1, QUALITY_ANY, FRAME_ANY));
	total_count = (total_count + UI_count_objects(PARTY, SHAPE_BODY1, QUALITY_ANY, FRAME_ANY));
	total_count = (total_count + UI_count_objects(PARTY, SHAPE_BODY2, QUALITY_ANY, FRAME_ANY));
	total_count = (total_count + UI_count_objects(PARTY, SHAPE_BODY3, QUALITY_ANY, FRAME_ANY));
	total_count = (total_count + UI_count_objects(PARTY, SHAPE_BODY4, QUALITY_ANY, FRAME_ANY));
	total_count = (total_count + UI_count_objects(PARTY, SHAPE_BODY5, QUALITY_ANY, FRAME_ANY));
	UI_error_message("greenCremation The Party has ", total_count, " of all movable bodies of any frame.");

	if (total_count == 0)
	{
		// Add a Party size check.
		say("@You and your companions do not appear to be carrying any bodies.@");
		//return;
	}
	
	///////////////////////////////////////////////////////////////////////////
	// Now we check the Party for each shape.
	UI_error_message("");

	body_count = UI_count_objects(PARTY, SHAPE_BODY1, QUALITY_ANY, FRAME_ANY);
	if (body_count >= 1)
	{
		UI_error_message("greenCremation The Party has ", body_count, " of SHAPE_BODY1.");
		body_400_shape = getPartyItems(SHAPE_BODY1, QUALITY_ANY, FRAME_ANY);
		UI_error_message("greenCremation SHAPE_BODY1 returns ", body_400_shape, ".");
		for (each in body_400_shape with index to max)
		{
			// Check for the relevant frames, even if rotated.
			frame = UI_get_item_frame(each);
			UI_error_message("greenCremation Frame ", frame, " found.");
			if (frame in body_400_frames)
			{
				// This check is necessary to prevent a non-existence "found" being counted by UI_get_array_size below.
				if (bodies == 0)
					bodies = each;
				else
					bodies = (bodies & each);
				UI_error_message("greenCremation SHAPE_BODY1, frame ", frame, " added to bodies array.");
			}
			else
				UI_error_message("greenCremation SHAPE_BODY1, frame ", frame, " was NOT added to bodies array.");
		}
		valid_bodies = UI_get_array_size(bodies);
		UI_error_message("greenCremation The burnable humanoid body count is now ", valid_bodies, ".");
	}
	else
		UI_error_message("greenCremation SHAPE_BODY1 was not found in the Party.");

	UI_error_message("");
	
	// Body 402
	body_count = (UI_count_objects(PARTY, SHAPE_BODY_402, QUALITY_ANY, FRAME_ANY));
	if (body_count >= 1)
	{
		UI_error_message("greenCremation The Party has ", body_count, " of SHAPE_BODY_402.");
		body_402_shape = getPartyItems(SHAPE_BODY_402, QUALITY_ANY, FRAME_ANY);
		for (each in body_402_shape with index to max)
		{
			// Check for the relevant frames, even if rotated.
			frame = UI_get_item_frame(each);
			UI_error_message("greenCremation Frame ", frame, " found.");
			if (frame in body_402_frames)
			{
				// This check is necessary to prevent a non-existence "found" being counted by UI_get_array_size below.
				if (bodies == 0)
					bodies = each;
				else
					bodies = (bodies & each);
				UI_error_message("greenCremation SHAPE_BODY_402, frame ", frame, " added to bodies array.");
			}
			else
				UI_error_message("greenCremation SHAPE_BODY_402, frame ", frame, " was NOT added to bodies array.");
		}
		valid_bodies = UI_get_array_size(bodies);
		UI_error_message("greenCremation The burnable humanoid body count is now ", valid_bodies, ".");
	}
	else
		UI_error_message("greenCremation SHAPE_BODY_402 was not found in the Party.");

	UI_error_message("");

	// Body 414
	body_count = (UI_count_objects(PARTY, SHAPE_BODY_414, QUALITY_ANY, FRAME_ANY));
	if (body_count >= 1)
	{
		UI_error_message("greenCremation The Party has ", body_count, " of SHAPE_BODY_414.");
		body_414_shape = getPartyItems(SHAPE_BODY_414, QUALITY_ANY, FRAME_ANY);
		for (each in body_414_shape with index to max)
		{
			// Check for the relevant frames, even if rotated.
			frame = UI_get_item_frame(each);
			UI_error_message("greenCremation Frame ", frame, " found.");
			if (frame in body_414_frames)
			{
				// This check is necessary to prevent a non-existence "found" being counted by UI_get_array_size below.
				if (bodies == 0)
					bodies = each;
				else
					bodies = (bodies & each);
				UI_error_message("greenCremation SHAPE_BODY_414, frame ", frame, " added to bodies array.");
			}
			else
				UI_error_message("greenCremation SHAPE_BODY_414, frame ", frame, " was NOT added to bodies array.");
		}
		valid_bodies = UI_get_array_size(bodies);
		UI_error_message("greenCremation The burnable humanoid body count is now ", valid_bodies, ".");
	}
	else
		UI_error_message("greenCremation SHAPE_BODY_414 was not found in the Party.");

	UI_error_message("");

	// Body 507
	body_count = (UI_count_objects(PARTY, SHAPE_BODY_SKELETON, QUALITY_ANY, FRAME_ANY));
	if (body_count >= 1)
	{
		UI_error_message("greenCremation The Party has ", body_count, " of SHAPE_BODY_SKELETON (507).");
		body_507_shape = getPartyItems(SHAPE_BODY_SKELETON, QUALITY_ANY, FRAME_ANY);
		for (each in body_507_shape with index to max)
		{
			// Check for the relevant frames, even if rotated.
			frame = UI_get_item_frame(each);
			UI_error_message("greenCremation Frame ", frame, " found.");
			if (frame in body_507_frames)
			{
				// This check is necessary to prevent a non-existence "found" being counted by UI_get_array_size below.
				if (bodies == 0)
					bodies = each;
				else
					bodies = (bodies & each);
				UI_error_message("greenCremation SHAPE_BODY_SKELETON (507), frame ", frame, " added to bodies array.");
			}
			else
				UI_error_message("greenCremation SHAPE_BODY_SKELETON (507), frame ", frame, " was NOT added to bodies array.");
		}
		valid_bodies = UI_get_array_size(bodies);
		UI_error_message("greenCremation The burnable humanoid body count is now ", valid_bodies, ".");
	}
	else
		UI_error_message("greenCremation SHAPE_BODY_SKELETON (507) was not found in the Party.");

	UI_error_message("");

	// Body 762
	body_count = (UI_count_objects(PARTY, SHAPE_BODY_762, QUALITY_ANY, FRAME_ANY));
	if (body_count >= 1)
	{
		UI_error_message("greenCremation The Party has ", body_count, " of SHAPE_BODY_762.");
		body_762_shape = getPartyItems(SHAPE_BODY_762, QUALITY_ANY, FRAME_ANY);
		for (each in body_762_shape with index to max)
		{
			// Check for the relevant frames, even if rotated.
			frame = UI_get_item_frame(each);
			UI_error_message("greenCremation Frame ", frame, " found.");
			if (frame in body_762_frames)
			{
				// This check is necessary to prevent a non-existence "found" being counted by UI_get_array_size below.
				if (bodies == 0)
					bodies = each;
				else
					bodies = (bodies & each);
				UI_error_message("greenCremation SHAPE_BODY_762, frame ", frame, " added to bodies array.");
			}
			else
				UI_error_message("greenCremation SHAPE_BODY_762, frame ", frame, " was NOT added to bodies array.");
		}
		valid_bodies = UI_get_array_size(bodies);
		UI_error_message("greenCremation The burnable humanoid body count is now ", valid_bodies, ".");
	}
	else
		UI_error_message("greenCremation SHAPE_BODY_762 was not found in the Party.");

	UI_error_message("");

	// Body 892
	body_count = (UI_count_objects(PARTY, SHAPE_BODY_SMALL, QUALITY_ANY, FRAME_ANY));
	if (body_count >= 1)
	{
		UI_error_message("greenCremation The Party has ", body_count, " of SHAPE_BODY_SMALL (892).");
		body_892_shape = getPartyItems(SHAPE_BODY_SMALL, QUALITY_ANY, FRAME_ANY);
		for (each in body_892_shape with index to max)
		{
			// Check for the relevant frames, even if rotated.
			frame = UI_get_item_frame(each);
			UI_error_message("greenCremation Frame ", frame, " found.");
			if (frame in body_892_frames)
			{
				// This check is necessary to prevent a non-existence "found" being counted by UI_get_array_size below.
				if (bodies == 0)
					bodies = each;
				else
					bodies = (bodies & each);
				UI_error_message("greenCremation SHAPE_BODY_SMALL (892), frame ", frame, " added to bodies array.");
			}
			else
				UI_error_message("greenCremation SHAPE_BODY_SMALL (892), frame ", frame, " was NOT added to bodies array.");
		}
		valid_bodies = UI_get_array_size(bodies);
		UI_error_message("greenCremation The burnable humanoid body count is now ", valid_bodies, ".");
	}
	else
		UI_error_message("greenCremation SHAPE_BODY_SMALL (892) was not found in the Party.");

	UI_error_message("");

	say("@A total of ", total_count, " bodies were found in your party.@");
	say("@Of these bodies, ", valid_bodies, " can be cremated.@");
}

/*
	bodies = UI_count_objects(PARTY, SHAPE_BODY1, QUALITY_ANY, FRAME_ANY);
	

	UI_get_array_size
	
	// First, check the party for bodies
	
	
	bodies = UI_count_objects(PARTY, SHAPE_BODY1, QUALITY_ANY, FRAME_ANY);
	UI_error_message("Glen has found ", bodies, ", in the party of Shape_Body1.");
	bodies = (bodies && UI_count_objects(PARTY, SHAPE_BODY1, QUALITY_ANY, FRAME_ANY));
	UI_error_message("Glen has found ", bodies, ", in the party of Shape_Body1 and Shape_Body2.");
	bodies = (bodies && UI_count_objects(PARTY, SHAPE_BODY3, QUALITY_ANY, FRAME_ANY));
	UI_error_message("Glen has found ", bodies, ", in the party of Shape_Body1 , Shape_Body2 and Shape_Body2.");
	bodies = (bodies && UI_count_objects(PARTY, SHAPE_BODY4, QUALITY_ANY, body_frames4)); //*should* select only from body frames listed above?
	UI_error_message("Glen has found ", bodies, ", in the party of Shape_Body1 , Shape_Body2 and Shape_Body2.");
	bodies = (bodies && UI_count_objects(PARTY, SHAPE_BODY5, QUALITY_ANY, FRAME_ANY));
	UI_error_message("Glen has found ", bodies, ", in the party of Shape_Body1 , Shape_Body2, Shape_Body3.");
	
			if (bodies > 0)
				{
				item.say("His eyes gleam fiendishly when he notices the body you're carrying.*");
				item.say("@Ah, you've brought me a client. Tell the truth, doesn't your friend look more peaceful this way?@*");
				item.say("@For such a heroic client, I have something special to offer. It's a new process I've invented called 'cremation.'@*");
				item.say("@For a mere fifteen gold, the remains of the dearly departed are purified with flame. Would you like me to do this for you?@*");
					if (askYesNo())
						{
						if (hasGold(15))
							{
							chargeGold(15);
							if (UI_find_object(PARTY, SHAPE_BODY1, QUALITY_ANY, FRAME_ANY) > 0)	{ body = UI_find_object(PARTY, SHAPE_BODY1, QUALITY_ANY, FRAME_ANY); }  //find body to cremate
							if (UI_find_object(PARTY, SHAPE_BODY2, QUALITY_ANY, FRAME_ANY) > 0) { body = UI_find_object(PARTY, SHAPE_BODY2, QUALITY_ANY, FRAME_ANY); } //find body to cremate
							if (UI_find_object(PARTY, SHAPE_BODY3, QUALITY_ANY, FRAME_ANY) > 0) { body = UI_find_object(PARTY, SHAPE_BODY3, QUALITY_ANY, FRAME_ANY); } //find body to cremate
							if (UI_find_object(PARTY, SHAPE_BODY4, QUALITY_ANY, body_frames4) > 0) { body = UI_find_object(PARTY, SHAPE_BODY4, QUALITY_ANY, body_frames4); } //find body to cremate								
							if (UI_find_object(PARTY, SHAPE_BODY5, QUALITY_ANY, FRAME_ANY) > 0) { body = UI_find_object(PARTY, SHAPE_BODY5, QUALITY_ANY, FRAME_ANY); } //find body to cremate
							//body = UI_find_object(PARTY, SHAPE_BODY2, QUALITY_ANY, FRAME_ANY); //find body to cremate, was supposd to pull from list
							//cremate_body = (cremate_body + body);
							//call crematorioum event	
							moveToLocation(body, body_pos); //for testing purposes, remove when finished! 
							
							
							}	
							else item.say("@You're a bit short on gold.@ he says, dissapointedly."); return; 
							
						}
						else item.say("@Too bad.@"); return;
				}
			else
	enum dead_body_shapes
{
	SHAPE_BODY1 = 400,					// 30 frames
		BODY_PRIESTESS = 3,					// Kylista's body
		BODY_UGLY_OLD_WOMAN = 4,			// Mosh's body
		BODY_FEMALE_PIKEMAN = 5,			// Shazzana's body
		BODY_FIGHTER_GREEN_HELM = 6,		// Brendann's body
		BODY_GWENNO = 8,					// Gwenno's body
		BODY_NOBLEMAN = 13,					// Marsten, Ardiniss
		BODY_MALE_RANGER_400 = 18,			// Used for post-Banes body outside of Frigidazzi's manor
		BODY_FEMALE_RANGER = 19,			// Julia's body
		QUALITY_ARDINISS_BODY = 124,		// Same as the brown scroll on his body
	SHAPE_BODY_402 = 402,					// 31 frames with SS
		BODY_FIGHTER_DARK_SKIN = 4,
		BODY_PIKEMAN = 11,
		BODY_MALE_RANGER = 12,				// Rangers that die in combat
	SHAPE_BODY_414 = 414,					// Dead 3 companions
		BODY_MALE_GOBLIN = 4,
		BODY_BATLIN = 16,
		BODY_FAWN_GUARD = 17,
		BODY_IOLO = 18,
		BODY_SHAMINO = 19,
		BODY_DUPRE = 20,
		BODY_GIRL_KNIGHT = 21,				// Cantra, matching below
	// SHAPE_BODIES_2 = 414,				// From original si_shapes.uc
	SHAPE_BODY_SKELETON = 507,				// Includes Gwani bodies
		BODY_GWANI_FULL_FUR = 14,
		BODY_GWANI_SOME_FUR = 15,
		BODY_GWANI_NO_FUR = 16,
	SHAPE_BODY_762 = 762,					// Mostly animals and monsters
	SHAPE_BODY_LARGE = 778,					// Larger monsters, dragons
	SHAPE_BODY_SMALL = 892,					// With glowing cat
		BODY_WOLF = 7
};

*/
