/*
 *
 *  Copyright (C) 2006  Alun Bestor/The Exult Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 *	This header file defines generic helper functions added to the original.
 *
 *	Author: Alun Bestor (exult@washboardabs.net)
 *	Modified by Exult Team
 *	Last modified: 2006-03-19
 */

//Generic functions
//-----------------

//returns a random entry from an array
var randomIndex(var array)
{
	var size = UI_get_array_size(array);
	var rand = UI_get_random(size);
	return array[rand];
}

var getSixth(var array)
{
	return array[6];

}





//checks to see if anyone in the party is poisoned 
var getPoisoned()
{
	var partymember;
	var party = partymember->get_party_list();
	var npc;
	var poisoned_party;
	var npcname;
	var npc_num;
	var party_member;
	var party_number;
	
	
	
	for (partymember in party with index)
	{
		
		UI_error_message("npc num is: " + npc_num);	
	//	UI_error_message("poisoned party is: " + npc);	
		if (UI_get_item_flag(npc, POISONED))
		{
			
			poisoned_party = npc->get_npc_object();
			return poisoned_party;
		}
		else
			return poisoned_party;
		UI_error_message("poisoned party is: " + npc);	
	}
	UI_error_message("Poisoned party member is: " + poisoned_party);
	
}

void freezeAvatar object# () ()
{
	UI_set_item_flag(AVATAR, BG_DONT_MOVE);


}

void unfreezeAvatar object# () ()
{
	UI_clear_item_flag(AVATAR, BG_DONT_MOVE);


}

//NPC-related functions
//---------------------

//Returns true if the specified object is the avatar, false otherwise.
var isAvatar(var object)	{ return (object->get_npc_object() == UI_get_avatar_ref()); }

//Check if the player has met the specified person (i.e. whether their Met flag has been set).
//Can take either an NPC constant or an object reference.
var hasMet(var npc)		{ return npc->get_item_flag(MET); }

//Returns true if the specified NPC is in the party, false otherwise
//There should be an intrinsic for this, but it doesn't appear to be defined
//Can take an NPC constant or an object reference.
var inParty(var npc)		{ npc = npc->get_npc_object(); return (npc in UI_get_party_list()); }

//quick bark/say functions, useful for testing. See also randomPartySay() and randomPartyBark() (original functions)
var avatarBark(var line)	{ if (canTalk(AVATAR)) AVATAR->item_say(line); }
var avatarSay(var line)		{ if (canTalk(AVATAR)) AVATAR.say(line); }


//Moving-stuff-around functions
//-----------------------------

//Shift an object into a container (can be from another container or the world)
var moveToContainer(var object, var container, var dont_check_ownership)
{
	var orig_pos;
	var orig_container;
	
	//record the previous container
	orig_container = object->get_container();
	//object was in the world - record its last position
	if (!orig_container) orig_pos = object->get_object_position();

	//could not remove the item (protected?)
	if (!object->set_last_created()) return false;

	//try to put the item into the new container
	if (container->give_last_created())
	{
		//check if the item was stolen, unless overridden
		if (!dont_check_ownership) stealItem(object);
		return true;
	}
	//if it couldn't be put into the new container, just put it back in the original container along with a mouse warning
	else
	{
		if (orig_container) { orig_container->give_last_created(); }
		else { UI_update_last_created(orig_pos); }
		
		//UI_flash_mouse(4);
		return false;
	}
}

//Opposite of moveToContainer: tries to place the item at the specified location, or returns it to its previous location/container if that couldn't be done.
var moveToLocation(var object, var pos)
{
	var orig_pos;
	var orig_container;
	
	//record the previous container
	orig_container = object->get_container();
	//object was in the world - record its last position
	if (!orig_container) orig_pos = object->get_object_position();

	//could not remove the item (protected?)
	if (!object->set_last_created()) return false;

	//try to shift the item
	if (UI_update_last_created(pos)) return true;

	//if it couldn't be moved, just put it back in the original container/location along with a mouse warning
	else
	{
		if (orig_container) { orig_container->give_last_created(); }
		else { UI_update_last_created(orig_pos); }
		
		UI_flash_mouse(CURSOR_WONT_FIT);
		return false;
	}
}


//generic get-item-from-container/world script, used by item interactions. It is used at the end of a march-to-container script, to play an appropriate animation and transfer the item into the player's inventory.
//once the item has been picked up, it will call the item's function with event = SCRIPTED.
void pickUpItem object#() ()
{
	var container = getOuterContainer(item);
	var direction = directionFromAvatar(container);

	//The avatar is already carrying the item, call the function immediately and leave it at that
	//This shouldn't happen, since gotoAndGet() will have picked up this case already - however, it could be the player drags the item into their inventory while walking, so...
	if (isAvatar(container))
	{
		//The brief delay is present in the original scripts, and is purely cosmetic.
		script item	{ nohalt; wait 1; call get_usecode_fun(); }
	}
	else
	{
		//container is an NPC, animate the avatar frobbing the target
		if (container->is_npc())
		{
			script AVATAR
			{	face direction;				actor frame ready;
				wait 3;						actor frame standing;}
		}
		//container is a regular object or just the world, animate the avatar leaning down
		else
		{
			script AVATAR
			{	face direction;				actor frame bowing;
				wait 3;						actor frame standing;}
		}

		//Puts the item in the player's inventory and calls the intended function
		script item
		{	nohalt;						wait 3;
			call giveToAvatar;			wait 2;
			call get_usecode_fun();}
	}
}


//Pathfind to the target item and pick it up once you get there (using pickUpItem). After this has happened, the item's function will be called with event = SCRIPTED. This is commonly used for item interactions that require the Avatar to be carrying the item first.
void gotoAndGet (var target)
{
	var offsetx;
	var offsety;
	var container;
	var func;

	UI_close_gumps();

	//item is contained by someone/thing - march the avatar over to the container and pick it up
	if (target->get_container())
	{
		container = getOuterContainer(target);
		//Avatar is the container - call the target's function immediately.
		if (isAvatar(container))
		{
			script target { nohalt; wait 1; call target->get_usecode_fun(); }
			return;
		}
		else
		{
			offsetx = [0, 1, -1, 1];
			offsety = [2, 1, 2, 0];

			gotoObject(container, offsetx, offsety, -3, pickUpItem, target, 2);
		}
	}
	//item was lying on the ground
	else
	{
		offsetx = [0, 1, 1, 1, -1, -1, 0, -1];
		offsety = [1, 1, 0, -1, 1, 0, -1, -1];

		//go to item and call result function
		//(func = pickUpItem())...which means I need to go and define bloody numbers again
		gotoObject(target, offsetx, offsety, -3, pickUpItem, target, 2);
	}
}


//Pathfind to the target item and pick it up once you get there (using pickUpItem). After this has happened, the item's function will be called with event = SCRIPTED. This is commonly used for item interactions that require the Avatar to be carrying the item first.
void gotoAndGet2 (var npc, var target)
{
	var offsetx;
	var offsety;
	var container;
	var func;

	UI_close_gumps();

	//item is contained by someone/thing - march the avatar over to the container and pick it up
	if (target->get_container())
	{
		container = getOuterContainer(target);
		//Avatar is the container - call the target's function immediately.
		if (isAvatar(container))
		{
			script target { nohalt; wait 1; call target->get_usecode_fun(); }
			return;
		}
		else
		{
			offsetx = [0, 1, -1, 1];
			offsety = [2, 1, 2, 0];

			gotoObject(container, offsetx, offsety, -3, pickUpItem, target, 2);
		}
	}
	//item was lying on the ground
	else
	{
		offsetx = [0, 1, 1, 1, -1, -1, 0, -1];
		offsety = [1, 1, 0, -1, 1, 0, -1, -1];

		//go to item and call result function
		//(func = pickUpItem())...which means I need to go and define bloody numbers again
		gotoObject(target, offsetx, offsety, -3, pickUpItem, target, 2);
	}
}






//returns true if obj is contained by target, or false otherwise
var containedBy (var obj, var target)
{
	var container;

	container = obj->get_container();
	while (container)
	{
		if (container == target) return true;
		container = container->get_container();
	}
	return false;
}


//Animation functions
//-------------------

//returns the direction (N/S/W/E) that the NPC is currently facing
var getFacing (var npc)
{
	var direction;
	var framenum;
	
	framenum = npc->get_item_frame_rot();

	if		(framenum >= EAST_FRAMESET)		direction = EAST;
	else if (framenum >= WEST_FRAMESET)		direction = WEST;
	else if (framenum >= SOUTH_FRAMESET)	direction = SOUTH;
	else direction = NORTH;

	return direction;
}


/*//DISABLED FOR NOW
void bellringer object#() ()
{
	var hour = UI_game_hour()-1;
	var minute = UI_game_minute();
	var frame = UI_get_item_frame(DAVER);
	var schedule = UI_get_schedule_type(DAVER);
	var bell = UI_find_nearest(item, SHAPE_BELL, 30); 


	if ((event == EGG) && (isNearby(DAVER)))
	{
		if (frame != 16)
			UI_set_item_frame(DAVER, 16);
	
		if (minute == 00)
		{
			script bell 
			{
				nohalt;
				frame 0;
				repeat hour //should repeat 1 time per hour
				{
					repeat 7
					{
						next frame cycle;
						
					};	
					sfx 19;
				};	
				wait 15;
				
			};
				
		}
		else
		{
			script item after 30 ticks
			{
			
				call bellringer;
			
			
			}
			
			
		
			
		}	
	}
	else
		abort;
			
	
}

*/


//Simple function to reverse a direction (NORTH becomes SOUTH, etc.)
var invertDirection (var direction)	{ return (direction + 4) % 8; }

//Gold-related functions (for streamlining shopping)
//--------------------------------------------------

//returns the total amount of gold the party has
var countGold (var amount)	{ return PARTY->count_objects(SHAPE_GOLD, QUALITY_ANY, FRAME_ANY); }

//returns true if the party has <amount> gold, false otherwise
var hasGold (var amount)
{
	var num_gold = PARTY->count_objects(SHAPE_GOLD, QUALITY_ANY, FRAME_ANY);
	return (num_gold >= amount);
}

//tries to deduct <amount> from the party's gold: returns true if they had the cash, or false if they can't afford it
var chargeGold (var amount)
{
	if (hasGold(amount)) return UI_remove_party_items(amount, SHAPE_GOLD, QUALITY_ANY, FRAME_ANY, true);
	else return false;
}

//give <amount> gold to the party: returns true if successful, false otherwise
var giveGold (var amount)	{ return UI_add_party_items(amount, SHAPE_GOLD, QUALITY_ANY, FRAME_ANY, true); }



//Script-related functions (used in script{} blocks)
//--------------------------------------------------
//Note that these functions cannot be passed any arguments, which is why they perform a specific action upon <item>.

//use during script sequences, to prevent the actor from moving according to schedule or player input
//IMPORTANT: Use nohalt; in these script sequences, otherwise the actor may remain frozen forever if the script is interrupted!
void GenericFreeze object#() ()	{ set_item_flag(BG_DONT_MOVE); }
void GenericUnfreeze object#() ()	{ clear_item_flag(BG_DONT_MOVE); }





/* 

void monsterStats (var monster, var stats)
{
	var stat_value;
	//IDs of the properties to set:
	var stat_ids = [STRENGTH, DEXTERITY, INTELLIGENCE, HEALTH, COMBAT];
	for (statnum in stat_ids)
	{
		//For each stat in the array, set the monster's attribute:
		stat_value = monster->get_npc_prop(statnum + 1);
		monster->set_npc_prop(statnum + 1, stats[statnum + 1] - stat_value);
	}
}

void monsterEquipment (var monster, var arms, var arms_quality, var pouch_content, var pouch_frames, var pouch_quantities, var pouch_qualities)
{
	var bag;
	var new_obj;
	var index;
	
//	for (itemnum in arms with counterindex)
	{
		//For each item in arms,
		//increment counter:
		//index = index + 1;
		//Create the object:
		new_obj = UI_create_new_object(itemnum);
		//Set quality:
		new_obj->set_item_quality(arms_quality[index]);
		//Give to monster:
		monster->give_last_created();
	}
	
	//Create a backpack:
	bag = UI_create_new_object(SHAPE_BACKPACK);
	//Give to monster:
	monster->give_last_created();
	//Reset counter:
	index = 0;
	while (index < UI_get_array_size(pouch_content))
	{
		//For each object in the array,
		//increment counter:
		index = index + 1;
		//Create object:
		new_obj = UI_create_new_object(pouch_content[index]);
		//Set frame:
		new_obj->set_item_frame(pouch_frames[index]);
		if (pouch_quantities[index] > 1)
			//Set quantity where available:
			new_obj->set_item_quantity(pouch_quantities[index]);
		//Set quality:
		new_obj->set_item_quality(pouch_qualities[index]);
		//Put in the backpack:
		bag->give_last_created();
	}
}

var monsterCreate (var shapenum, var npcid, var pos, var facedir, var stats, var arms, var arms_quality, var pouch_content, var pouch_frames, var pouch_quantities, var pouch_qualities)
{
	//Create the monster:
	var monster = UI_create_new_object(shapenum);
	//Initially set monster to be neutral:
	monster->set_alignment(1);
	//Have the monster be in WAIT mode:
	monster->set_schedule_type(WAIT);
	//Make him temporary:
	monster->set_item_flag(TEMPORARY);
	
	
	//Set the monster's attributes:
	monsterStats(monster, stats);
	//Give him some equipment:
	monsterEquipment(monster, arms, arms_quality, pouch_content, pouch_frames, pouch_quantities, pouch_qualities);
	//Set NPC ID to requested value:
	monster->set_npc_id(npcid);

	
	//Set monster as last created:
	monster->set_last_created();
	//Update monster to desired spot:
	UI_update_last_created(pos);
	//Have monster face desired direction:
	script monster face facedir;
	return monster;
}


*/

//Functions added by DB//
//---------------------------------------------------

var isWearingAmulet()
{
	if (UI_is_readied(AVATAR, BG_AMULET, SHAPE_AMULET_SUBMISSION, FRAME_ANY))
		return true;
	else
		return false;

}




var throwRandomBottle(var target) //location passed to function
{
	var rand_bottle =  UI_die_roll(6, 12); //for bottle frames
	var rand_booze =  UI_die_roll(8, 15); //for booze frames 
	
	var bottle = UI_create_new_object(SHAPE_BOTTLE);
	bottle->UI_set_item_frame(rand_bottle); 
	bottle->update_last_created([target]);


	var booze = UI_create_new_object(912); //for booze splotch (blood shape)
	booze->set_item_frame(rand_booze);
	booze->update_last_created([target]);
	UI_play_sound_effect(37);
	
	
}


var getExpLevel(var npc)
{
	var level;
	var current_experience = UI_get_npc_prop(npc, EXPERIENCE);
	
	if ((current_experience > 0) && (current_experience < 100))
	{
		level = 1;
		return level;
	}
	else
	if ((current_experience >= 100) && (current_experience < 200))
	{
		level = 2;
		return level; 
	}
	else
	if ((current_experience >= 200) && (current_experience < 400))
	{
		level = 3;
		return level; 
	}
	else	
	if ((current_experience >= 400) && (current_experience < 800))
	{
		level = 4;
		return level; 
	}	
	else	
	if ((current_experience >= 800) && (current_experience < 1600))
	{
		level = 5;
		return level; 
	}	
	else
	if ((current_experience >= 1600) && (current_experience < 3200))
	{
		level = 6;
		return level; 
	}
	else
	if ((current_experience >= 3200) && (current_experience < 6400))
	{
		level = 7;
		return level; 
	}
	else	
	if (current_experience >= 6400)
	{
		level = 8; //no spells above 8 so it doesn't matter
		return level; 
	}
	
	
	
	}

var hasSpell(var spell) //used to check if certain spell in spellbook in AVATAR's inventory
{
	var spellbook = UI_find_object(AVATAR, 761, QUALITY_ANY, FRAME_ANY); //makes sure Avatar has spellbook
	
	return UI_has_spell(spellbook, spell);
	
}




var daypart(var part_of_day)
{
	
	var hour = UI_game_hour();
	
	if ((hour >= 6) && (hour < 12)) //'good morning'
	{
		return "morning";
		//return time_of_day;
	} 
	else if ((hour >= 12) && (hour < 18)) //'good afternoon'
	{
		part_of_day = "afternoon";
		return "afternoon";
	}
	else
		part_of_day = "evening";
	return part_of_day;
	
}


  
void Sank object# () ()
{
	var raft = UI_find_nearby_avatar(SHAPE_RAFT);
	var barge = get_barge();
	var npc; 
	var party = UI_get_party_list();
	
	UI_halt_scheduled(raft); //should stop any other scripts from running
	
	UI_remove_item(raft);
	UI_remove_item(barge);
	

	
	UI_clear_item_flag(AVATAR, SAILOR);
	UI_clear_item_flag(AVATAR, ON_MOVING_BARGE);
	UI_clear_item_flag(AVATAR, IN_MOTION);
	
	UI_kill_npc(AVATAR);
	UI_clear_item_flag(AVATAR, BG_DONT_MOVE);
	
	UI_set_npc_id(SMITH, 0); //resets SMITH's ID for warning level 0. 
	
}

void Sinking object# () ()
{
	var raft = UI_find_nearby_avatar(SHAPE_RAFT);
	var barge = get_barge();
	var npc; 
	var party = UI_get_party_list();
	
	
	UI_set_item_flag(AVATAR, BG_DONT_MOVE);
	UI_halt_scheduled(raft);
	
	UI_earthquake(40); //change if need be
	
	randomPartyBark("@We're sinking!!@");
	
	for (npc in party)
	{
		script npc after 3 ticks	
		{
			repeat 2
			{
				say "@Aiieeee!@"; wait 2; actor frame bowing; wait 1; actor frame kneeling; wait 1;
				actor frame standing; wait 2; actor frame LIE; 
				actor frame LIE; wait 1;
			};
		};
	
	}	
	
	script raft after 20 ticks
	{
		sfx 25; wait 5; sfx 38; wait 1; sfx 25; wait 3; 
		sfx 26; wait 3; sfx 26; wait 4; sfx 38; wait 2; 
		sfx 40; wait 3; sfx 38; wait 2; sfx 40; wait 5;
						
		call Sank;
	};
	
	
	
}


	
void DeepWater object#() () //used to determine if raft is too far away from shore - in deep water. 
{
	var barge = get_barge();
	var surf = [612, 613, 632, 699, 736, 737, 751, 806, 834, 875, 907, 911, 918];
	var waves = [384, 985, 1008, 1009];
	var warning = UI_get_npc_id(SMITH); //warning level for raft. If too much time in deep water, raft will sink. 
	var raft = UI_find_nearby_avatar(SHAPE_RAFT);
	
	UI_error_message("Deep water script called"); //remove after testing
	UI_error_message("@Warning level is " + warning + "@");
	
	if (warning >= 4)
	{
		script barge after 10 ticks
		{
			call Sinking;
		};
	}

	if (UI_find_nearby(AVATAR, waves, 7, 0x10))
	{
		UI_set_npc_id(SMITH, warning + 1); //sets ID for warning level. 
		UI_error_message("Waves are found"); //for testing
		//add various levels of warnings based on warning level. 
		if (warning == 1)
			randomPartySay("@This raft isn't meant for deep ocean travel, we must turn back to shore!@"); 
		if (warning == 2)
			randomPartySay("@We must turn back now, the raft will break upon these waves!@");
		if (warning == 3)
			randomPartySay("@This raft isn't meant for deep ocean travel, we must turn back!@");
		if (warning >= 4)
		{
			randomPartySay("@The raft is breaking apart! We're going to sink!!@");
		
		}

	}
	script raft after 60 ticks	
	{
		call DeepWater;

	}
	
	
}


void Eruption (var egg) 
{
	const int SPRITE_EXPLOSION = 1;
	const int SFX_ERUPT = 60;
	var avatar_distance = UI_get_distance(egg, AVATAR);

	//determine if it explodes
	var rand = UI_get_random(100);
	UI_error_message("Volcano egg hatched: rand number is " + rand);
	
	if (rand < 25)
	{
		UI_error_message("Rand numb was " + rand + ", volcano erupts");
		var target; 
		
	
		//determine how many npcs are neaby
		 var targets = UI_find_nearby(egg, SHAPE_ANY, 6, MASK_NPC2);
		
		//exp effect
	
		UI_obj_sprite_effect(egg, SPRITE_EXPLOSION, 0, 0, 0 ,0, 0, -1);
		
		//play sound
		UI_play_sound_effect(SFX_ERUPT, item);
		
		UI_earthquake(1);
		var count = 0; 
		 //apply damage
		 for (target in targets)
		 {
			count += 1;
			UI_error_message("Target  " + count + ", is damaged!");
			var target_distance = UI_get_distance(egg, target);
			var damage_base = (50 / target_distance);
			UI_apply_damage(damage_base, 15, FIRE_DAMAGE, target);
		 }
			
		
	}
//	script egg after 50 ticks
//	{
//		call volcano;
//	}
	
}






extern var getAvatarName 0x0908();
extern var getNPCName 0x90F (var npc);
extern var getNPCStat 0x910 (var npc, var stat);
extern void trainStrength 0x914 (var npc, var delta);
extern void trainDexterity 0x915 (var npc, var delta);
extern void trainIntelligence 0x916 (var npc, var delta);
extern void trainCombat 0x917 (var npc, var delta);
extern void trainMagic 0x918 (var npc, var delta);
extern var selectWhoTrains 0x920 ();
extern var checkCanTrain 0x922 (var statArray, var goldCost, var npc, var numTraining);

void LoubetTrain 0x8D0 (var statArray, var goldCost)
{
	var pronoun;
	var npc = selectWhoTrains();
	var name = getNPCName(npc);
	var verb = "feels";
	var avname = getAvatarName();
	var pronoun2;
	var title;
	var possessive;
	if (name == avname)
	{
	
	if (!gflags[TRAINED_WITH_LOUBET])
	{
		gflags[TRAINED_WITH_LOUBET] = true;
	}
		name = "you";
		pronoun = "you";
		pronoun2 = "you";
		verb = "feel";
	}
	else
	{
		pronoun2 = "they";
		if (npc == JULIA || npc == JAANA|| npc == KATRINA || npc == LEODON || npc == LEONNA || npc == GWENNO || npc == KATRINA)
		{
			pronoun = "her";
			title = "madame";
			possessive = "she's";
		}
		else
		{
			pronoun = "him";
			title = "m'sieur";
			possessive = "he's";
		}
	}
	if (npc)
	{
		var ret = checkCanTrain(statArray, goldCost, npc, 3); //train points?
		if (ret == 0)
			say("@I am sorry, but thou dost not have enough experience to train at this time. Return at a later date and zen I would be most happy to give thee a lesson.@");
		else if (ret == 1)
		{
			var partyGold = UI_count_objects(PARTY, SHAPE_GOLD, QUALITY_ANY, FRAME_ANY);
			if (partyGold < goldCost)
				say("@Mon dieu! It seems that thou hast not enough gold. Do return when thou art a bit wealthier.@");
		}
		else if (ret == 2)
			say("@But no! You are too skilled, I 'ave nothing more to teach you!@"); //too skilled
		else
		{
			UI_remove_party_items(goldCost, SHAPE_GOLD, QUALITY_ANY, FRAME_ANY, true);
			say("Loubet takes your money then hands " + name + " a foil and a mask of golden mesh.*");
			say("He shows " + name + " the correct stance, then " + pronoun2 + " spar for a bit.*");
			say("Now ze off hand, " + title + "... Mon dieu, you are -how you say it?- ambidextrous! Very good, mah friend.@");
			say("M'sieur Loubet is a skilled teacher, and " + name + " " + verb + " like " + possessive + " really learned something.");


			if (getNPCStat(npc, DEXTERITY) < 30)
				trainDexterity(npc, 1);

			if (getNPCStat(npc, COMBAT) < 30)
				trainCombat(npc, 2); //
		}
	}
}




